/*
	File:			CAFileUtils.cpp

	Contains:	File utility implementations

	Written by:	Rick Badertscher

	Copyright:	 1993-95 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <0>	  7/20/95	RB		Created


	To Do:
*/

#ifndef _EXCEPT_
#include <Except.h>
#endif

#ifndef SOM_Module_OpenDoc_StdDefs_defined
#include <StdDefs.xh>
#endif

#ifndef _PLFMFILE_
#include <PlfmFile.h>
#endif

#ifndef _PASCLSTR_
#include <PasclStr.h>
#endif

#ifndef _CAFILEUTILS_
#include "CAFileUtils.h"
#endif

#ifndef _CAUTIL_
#include "CAUtil.h"
#endif

#ifndef _CASESSN_
#include "CASessn.h"
#endif

#ifndef SOM_ODDocument_xh
#include <Document.xh>
#endif

#ifndef SOM_ODContainer_xh
#include <ODCtr.xh>
#endif

#ifndef _STORUTIL_
#include <StorUtil.h>
#endif

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif

#ifndef _DOCUTILS_
#include <DocUtils.h>
#endif

#ifndef _TEMPOBJ_
#include <TempObj.h>
#endif

#ifndef _ODPRCS_
#include "ODPrcs.h"
#endif

//==============================================================================
// Constants
//==============================================================================

#define kMaxNumberSuffixLength 10
#define	kSIZERsrcType		'SIZE'
const short kMaxUniqueNameTries = 100;

#define kODErrDirectoryNotHaveWriteAccess  		-29824
#define kODErrSystemVolumeIsFull   				-29821

typedef short ForkType;
typedef short *RefNumPtr;


//==============================================================================
// Function Prototype
//==============================================================================

ODStatic short gUniqueNameSeed = 1;

ODStatic void	CreateNewUntitledFile(	PlatformFile* newFile);
ODStatic void	CreateNewTempFile(		PlatformFile* newFile);
ODStatic void	CreateNewFile(			PlatformFile* newFile);

ODStatic pascal	OSErr	GetLocalPaths(	FSSpec *spec,
							 			ForkType fork,
							  			unsigned long *refNumCount,
							  			RefNumPtr *refNums);
ODStatic void		CreateUniqueTmpFolderForFile(ODFileSpec* result, Str63 fileName);
ODStatic void		GetTempFolder(ODSShort* VRef,ODSLong* DirID);
ODStatic void		CreateTempName(short seed, Str255 name, ODBoolean isFile);

//-------------------------------------------------------------------------------------

PlatformFile*	CreateTempContainer(Environment* ev, ODSession* session, ODDocument** documentPtr,
			ODContainer** containerPtr)
{
	PlatformFile*	file;
	
	file = CreateTempFile( kODShellSignature, kODShellSignature, kODNULL );

	(*containerPtr) = CreateFileContainer( ev, session, &(file->GetFileSpec()));
	
	if (documentPtr)
		(*documentPtr) = (*containerPtr)->AcquireDocument(ev, kODDefaultDocument);
		
	return file;
	
}

//-------------------------------------------------------------------------------------

PlatformFile* CreateTempFile(ODOSType creator, ODOSType fileType, ODName* fileNameSeed)
{
	PlatformFile*	newFile;
	Str255			name;
	ODFileSpec		spec;
	
	char* cfnSeed = kODNULL;
	if (fileNameSeed != kODNULL)
		cfnSeed = GetITextCString(fileNameSeed, kODNULL);
		
	CreateTempName(0, name, kODTrue);
	
	CreateUniqueTmpFolderForFile(&spec, name);

	newFile = new PlatformFile;
	newFile->Specify (&spec);
	//CreateNewTempFile(newFile);
	
	newFile->Create(creator, fileType, 0);

	return (newFile);


}

//-------------------------------------------------------------------------------------

void	CreateUntitledContainer(Environment* ev, ODSession* session, ODDocument** documentPtr,
			ODContainer** containerPtr, PlatformFile* newFile, ODName* fileNameSeed)
{
	char* cfnSeed = kODNULL;
	if (fileNameSeed != kODNULL)
		cfnSeed = GetITextCString(fileNameSeed, kODNULL);
		
	CreateNewUntitledFile(newFile);

	ODDeleteObject(cfnSeed);
	
	newFile->Create(kODShellSignature, kODShellSignature, 0);

	ODULong refNumCount;
	ODSShort* refNums;
	if (documentPtr)
	{
		GetLocalPaths(&(newFile->GetFileSpec()), kDataFork,
						  &refNumCount, &refNums);
	}

	(*containerPtr) = CreateFileContainer(ev, session, &(newFile->GetFileSpec()));

	if (documentPtr)
	{
		*documentPtr = (*containerPtr)->AcquireDocument(ev, kODDefaultDocument);	// -- T: documentPtr acquires a document
	}
}

//-------------------------------------------------------------------------------------
// New File(Name/Spec)
//-------------------------------------------------------------------------------------

ODStatic void	CreateNewUntitledFile(PlatformFile* newFile)
{
	ODULong	secs;
	GetDateTime(&secs);
	short	index = 1;
	ODSByte	indexString[5];
	ODSByte	tmpString[kODMaxFileNameSize];

	
	ODSByte	dateString[kODMaxFileNameSize];
	IUDateString(secs,shortDate,(StringPtr)dateString);
	PascalToCString((StringPtr)dateString);

	ODSByte	untitledName[kODMaxFileNameSize];
	strcpy(untitledName,"untitled ");
	strcat(untitledName,dateString);
	strcat(untitledName," ");

	do
	{
	
		strcpy (tmpString, untitledName);
		NumToString (index, (StringPtr) indexString);
		PascalToCString((StringPtr)indexString);

		strcat(tmpString, indexString);
		newFile->SetAsciiName(tmpString);
		++index;
		
	} while (newFile->Exists());
	
	CreateNewFile(newFile);
}
//-------------------------------------------------------------------------------------

ODStatic void	CreateNewTempFile(PlatformFile* newFile)
{
	ODULong	secs;
	GetDateTime(&secs);
	short	index = 1;
	ODSByte	indexString[5];
	ODSByte	tmpString[kODMaxFileNameSize];
	ODSByte	tmpName[kODMaxFileNameSize];

	ODSByte	dateString[kODMaxFileNameSize];
	IUDateString(secs,shortDate,(StringPtr)dateString);
	PascalToCString((StringPtr)dateString);

	strcpy(tmpName,"tmp ");
	strcat(tmpName,dateString);
	strcat(tmpName," ");

	do
	{
	
		strcpy (tmpString, tmpName);
		NumToString (index, (StringPtr) indexString);
		PascalToCString((StringPtr)indexString);

		strcat(tmpString, indexString);
		newFile->SetAsciiName(tmpString);
		++index;

	} while (newFile->Exists());


	CreateNewFile(newFile);
}


//-------------------------------------------------------------------------------------

ODStatic void CreateUniqueTmpFolderForFile(ODFileSpec* result,
		Str63 fileName)
{
	Str255		myName;
	ODSLong	myDirID;
	ODSShort	myVRef;
	GetTempFolder(&myVRef, &myDirID);

	OSErr err;
	short limit = kMaxUniqueNameTries;
	while ( --limit )
	{
		CreateTempName( gUniqueNameSeed++, myName, kODFalse );
		long createdDirID;
		err = DirCreate( myVRef, myDirID, myName,
				&createdDirID );
		if ( err == noErr )
		{
			FSMakeFSSpec(myVRef, createdDirID, fileName, result);
			return;
		}
	}
	THROW( err );
}

//-------------------------------------------------------------------------------------
ODStatic void	GetTempFolder(ODSShort* VRef,ODSLong* DirID)
{
	FindFolder(0, kTemporaryFolderType, kCreateFolder,VRef,DirID);
}


//-------------------------------------------------------------------------------------

ODStatic void	CreateTempName(short seed, Str255 name, ODBoolean isFile)
{
	// Don't modify this one!
	char* nmSeed = isFile? "CALibTempFile" : "CALibTempFolder";
	strcpy((char*)name, nmSeed);

	char theNum[10];
	NumToString(seed, (StringPtr)theNum); 
	WASSERT(theNum[0] <= 5);
	p2cstr((StringPtr)theNum);

	strcat((char*)name, theNum);
	c2pstr((char *)name);
}


//-------------------------------------------------------------------------------------

ODStatic void	CreateNewFile(PlatformFile* newFile)
{
	ODSLong		seed = 0;
	ODULong		err = noErr;

	ODSByte	 	prefixName[kODMaxFileNameSize+1];
	newFile->GetAsciiName(prefixName, kODMaxFileNameSize);
	prefixName[kODMaxFileNameSize] = 0;
	ODUShort	prefixLen = strlen(prefixName);

	ODSByte	 	newName[kODMaxFileNameSize+1];
	strcpy(newName,prefixName);
	
	ODFileSpec fs = newFile->GetFileSpec();

	while (GetVolumeWritableSpace(&fs, kODNULL) < kSpaceNeededForNewDocument)
	{
		
		THROW_IF_ERROR(FindFolder(kOnSystemDisk, kDesktopFolderType, kCreateFolder,
								&(fs.vRefNum), &(fs.parID)));
		
		if (GetVolumeWritableSpace(&fs, &err) >= kSpaceNeededForNewDocument)
	  		break;

		THROW_IF_ERROR(err);
		THROW(kODErrSystemVolumeIsFull);

	// It would be nice to try other volumes as well, like:
	//   iterate through local non-removable volumes
	//     if enough space on one of those, then use it & break; break;
	//   iterate through local removable volumes
	//     if enough space on one of those, then use it & break; break;
	//   iterate through remote volumes
	//     if enough space on one of those, then use it & break; break;
	
	//  THROW(kODErrAllWritableVolumesAreFull);

	}
	newFile->Specify(&fs);	
	
	do {
		if (seed++>0) // May want to change this to ++seed if we eliminate the Date from the prefix
		{
			ODSByte theNum[kMaxNumberSuffixLength];
			
			NumToString(seed, (StringPtr)theNum); // Macintosh Specific
			PascalToCString((StringPtr)theNum);
			ODUShort maxPrefixLen = kODMaxFileNameSize - 1 - strlen(theNum);
			if (prefixLen > maxPrefixLen)
			{
				prefixLen = maxPrefixLen;
				prefixName[prefixLen] = 0;
			}
			strcpy(newName, prefixName);
			strcat(newName, " ");
			strcat(newName, theNum);
		}
		newFile->SetAsciiName(newName);
	} while (newFile->Exists());
}	// CreateNewFile


//-------------------------------------------------------------------------------------

ODStatic pascal	OSErr	GetLocalPaths(FSSpec *spec,
							  ForkType fork,
							  unsigned long *refNumCount,
							  RefNumPtr *refNums)
{
	enum
	{
		kMaxFCBs = 348	/* The maximum number of local access paths under System 7 */
	};
	
	OSErr		result;
	FCBPBRec	pb;
	short		index;
	Str31		tempName;
	RefNumPtr	tempRefNums;
	
	/* initialize returned values */
	*refNumCount = 0;
	*refNums = NULL;
	
	/* Allocate temp storage for refNum matches */
	tempRefNums = (RefNumPtr)NewPtr(kMaxFCBs * sizeof(short));
	
	if ( tempRefNums != NULL )
	{
		/* Get FCB name in tempName */
		pb.ioNamePtr = tempName;
		
		/* Index through the open paths on the volume specified by spec->vRefNum */
		pb.ioVRefNum = spec->vRefNum;
		index = 1;
		do
		{
			pb.ioRefNum = 0;
			pb.ioFCBIndx = index;
			result = PBGetFCBInfoSync(&pb);
			if ( result == noErr )
			{
				/*
				**	See if parent directory ID matches and
				**	file name matches and
				**	the file fork (resource or data) matches
				*/
				if ( (pb.ioFCBParID == spec->parID) &&
					 EqualString(spec->name, tempName, false, true) &&
					 ((fork == kResourceFork) == ((pb.ioFCBFlags & (1 << 9)) != 0)) )
				{
					/* It's a match - add it to the array of matches */
					tempRefNums[*refNumCount] = pb.ioRefNum;
					++(*refNumCount);
				}
				++index;	/* next FCB index */
			}
		} while ( result == noErr );
		
		/* These errors are OK - they mean we hit the end of the FCB list */
		if ( (result == rfNumErr) || (result == fnOpnErr) )
		{
			result = noErr;
		}
		
		if ( (result == noErr) && (*refNumCount != 0) )
		{
			/* Allocate memory to return refNum matches in */
			*refNums = (RefNumPtr)NewPtr(*refNumCount * sizeof(short));
			if ( *refNums != NULL )
			{
				/* Move refNums from tempRefNums to *refNums */
				BlockMoveData((Ptr)tempRefNums, (Ptr)*refNums, (*refNumCount * sizeof(short)));
			}
			else
			{
				/* Couldn't allocate memory for refNum array */
				result = memFullErr;
			}
		}
		
		/* Free up tempRefNums storage */ 
		DisposePtr((Ptr)tempRefNums);
	}
	else
	{
		/* Couldn't allocate temp memory */
		result = memFullErr;
	}
	
	return ( result );
}


PlatformFile* CreatePartFromStationary (PlatformFile* file, ODPart** part)
{
	Environment*	ev			= gCASession->GetEV();
	ODContainer*	sContainer	= kODNULL;
	ODDocument*		sDocument	= kODNULL;
	ODContainer*	nContainer	= kODNULL;
	ODDocument*		nDocument	= kODNULL;
	ODDraft*		sDraft		= kODNULL;
	ODDraft*		nDraft		= kODNULL;
	PlatformFile*	newFile		= kODNULL;
	
	ODVolatile(sContainer);
	ODVolatile(sDocument);
	ODVolatile(sDraft);
	
	
	TRY
	
		ODAcquireCtrDocTopDraft(ev, gCASession->GetODSession(), file, &sContainer, &sDocument, &sDraft);
	
		newFile = CreateTempContainer(ev, gCASession->GetODSession(), &nDocument,
									 &nContainer);
		
		{
		
			CUsingLibraryResources r;
			VerifyDocumentStub(&(newFile->GetFileSpec()), kODNULL);
			
			// Makes sure to copy resources into the new document.
					
			ODULong sizeOfResource;
			ODPtr sizeRes = file->ReadResourcePtr(kSIZERsrcType, 0, &sizeOfResource);
			if (sizeRes != kODNULL)
				newFile->WriteResourcePtr(kSIZERsrcType, 0, sizeRes, sizeOfResource);
			ODDisposePtr(sizeRes);
			sizeRes = file->ReadResourcePtr(kSIZERsrcType, 1, &sizeOfResource);
			if (sizeRes != kODNULL)
				newFile->WriteResourcePtr(kSIZERsrcType, 1, sizeRes, sizeOfResource);
			ODDisposePtr(sizeRes);
		}
		
		nDocument->SetBaseDraftFromForeignDraft(ev, sDraft);			
		//ODResetDateModByInfo(ev, nDocument);
				
		ODReleaseObject(ev, sDraft);		// -- T: release sDraft in the TRY
		ODReleaseObject(ev, sDocument);	// -- T: release sDocument in the TRY
		ODReleaseObject(ev, sContainer);	// -- T: release sContainer in the TRY
				
		newFile->SetPlatformType(file->GetPlatformType());

		ODDocumentName* name = newFile->GetName();
		nDocument->SetName(ev, name);
	
		nDraft = nDocument->AcquireBaseDraft(ev, kODDPExclusiveWrite);
		TempODStorageUnit su = nDraft->AcquireDraftProperties(ev);
		ODResetDateModByInfo(ev, su);

		*part = ODAcquireRootPartOfDraft (ev, nDraft);
		
	CATCH_ALL
	ENDTRY
	
	return (newFile);
	
		
}




