/*
	File:		CAFrmUtl.cpp

	Contains:	Container Application Library private frame utility source

	Written by:	Rick Badertscher, Tantek elik

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <16>	 11/10/95	RB		Modified FocusDocForFrameRef() to focus on
		 							kCAPropEmbeddedFrames property, instead of 
		 							first contents property
		 <15>	 5/8/95		SJF		Add include of ODMemory.h for ODBlockMove()
		 <13>	 5/7/95		RB		Modified FrameFromFrameRef() and FrameRefFromFrame()
		 							to handle the root frame as a special case.
		 							
		 							Added routines FrameRefIsRoot() & FrameRefSetRoot().
		 							
		 							Fixed reference count bug in GetRootFrame()
		 							
		 <9>	 4/23/95	RB		Added ZeroFrameRef()
		 							Added CAFrameRefValid(), CAFrameRefEqual(),
		 							CAFrameRefCopy()
		 							
		 <8>	 4/06/95	RB		Modified GetRootFrame()
		 <7>	 4/04/95	RB		Fixes for b1c14.
		 <6>	 3/31/95	RB		Added failure handling
		 <5>	 3/30/95	RB		API cleanup
		 <4>	 3/10/95	RB		Added FacetFromFrameRef
		 <3+>	 2/28/95	SJF		Add debug stuff to API calls, merge in changes from RickB
		 <3>	 2/13/95	SJF		Interim checkin to update project database
		 <2)	12/15/94	SJF		change somGetGlobalEnvironment to _gpProxyShell->GetGlobalEnvironment
		 <1>	10/30/94	GCA,DHN	All mods to make project compile with no
		 							errors under OpenDoc b1 (with SOM).
		 <0>	  8/9/94	T		first written
		 
	To Do:
*/

#ifndef _CAFRMUTL_
#include "CAFrmUtl.h"
#endif

#ifndef _CADOCPRIV_
#include "CADocPriv.h"
#endif

#ifndef _CASESSN_
#include "CASessn.h"
#endif

#ifndef SOM_Module_OpenDoc_Global_Types_defined
#include <ODTypesM.xh>
#endif

#ifndef _STORAGU_
#include <StorageU.xh>
#endif

#ifndef SOM_ODFrame_xh
#include <Frame.xh>
#endif

#ifndef SOM_ODDraft_xh
#include <Draft.xh>
#endif

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif

#ifndef SOM_ODFrameFacetIterator_xh
#include <FrFaItr.xh>
#endif

#ifndef SOM_ODWindowIterator_xh
#include <WinIter.xh>
#endif

#ifndef SOM_ODWindowState_xh
#include <WinStat.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdTypes_defined
#include <StdTypes.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdDefs_defined
#include <StdDefs.xh>
#endif

#ifndef SOM_ODWindow_xh
#include <Window.xh>
#endif

#ifndef _STORUTIL_
#include <StorUtil.h>
#endif

#ifndef SOM_ODStorageUnitRefIterator_xh
#include <SURefItr.xh>
#endif

#ifndef _TEMPOBJ_
#include <TempObj.h>
#endif

#pragma segment CALib

static void			FocusDocForFrameRef (CADocumentRef document);

//-------------------------------------------------------------------------

CAFrameRef	GetFrameRefFromID(CADocumentRef document, ODID frameID)
{

	Environment*		ev = gCASession->GetEV();
	ODStorageUnit*		rootPartSU	= kODNULL;
	ODDraft*			theDraft	= kODNULL;
	CAFrameRef			frameRef;
	ODStorageUnitRef	suRef;
	
	rootPartSU = ((CADocument*)document)->GetRootPartSU();
	THROW_IF_NULL (rootPartSU);
		
	theDraft = rootPartSU->GetDraft (ev);

	// Focus su on value where the framerefs will be stored
	FocusDocForFrameRef (document);
	rootPartSU->GetStrongStorageUnitRef (ev, frameID, suRef);
	
	ODBlockMove ((ODPtr) suRef, (ODPtr) &frameRef, 4);
	return (frameRef);

}

//-------------------------------------------------------------------------

ODID	GetIDFromFrameRef(CADocumentRef document, CAFrameRef frameRef)
{

	Environment*		ev = gCASession->GetEV();
	ODStorageUnit*		rootPartSU = kODNULL;
	ODDraft*			theDraft = kODNULL;
	ODStorageUnitRef	suref;
	ODFrame*			frame = kODNULL;
	ODID				frameID;
	
	rootPartSU = ((CADocument*)document)->GetRootPartSU();
	THROW_IF_NULL (rootPartSU);
		
	theDraft = rootPartSU->GetDraft (ev);

	// Focus su on value where the framerefs will be stored
	FocusDocForFrameRef (document);
	
	ODBlockMove ( (ODPtr) &frameRef, (ODPtr) suref, 4);
	frameID = rootPartSU->GetIDFromStorageUnitRef (ev, suref);

	return (frameID);

}
//-------------------------------------------------------------------------

ODFrame*	AcquireFrameFromFrameRef(CADocumentRef document, CAFrameRef frameRef)
{
	Environment*		ev			= gCASession->GetEV();
	ODFrame*			frame		= kODNULL;
	ODStorageUnit*		rootPartSU;
	ODStorageUnitRef	suref;
	ODID				frameID;
	
	if (frameRef == kRootFrameRef)
	{
		frame = GetRootFrame(document);
		ODAcquireObject (ev, frame);
		return (frame);
	}

	rootPartSU = ((CADocument*)document)->GetRootPartSU();
	THROW_IF_NULL (rootPartSU);
	
	// Focus su on value where the framerefs will be stored
	FocusDocForFrameRef (document);
	
	ODBlockMove ( (ODPtr) &frameRef, (ODPtr)suref, 4);
	frameID = rootPartSU->GetIDFromStorageUnitRef (ev, suref);
	frame = rootPartSU->GetDraft(ev)->AcquireFrame (ev, frameID);
	
	THROW_IF_NULL (frame);

	return (frame);
				
}

//-------------------------------------------------------------------------

CAFrameRef	FrameRefFromFrame(	CADocumentRef	document,
								ODFrame*		frame)
{
	Environment*		ev			= gCASession->GetEV();
	ODStorageUnit*		rootPartSU	= kODNULL;
	ODID				frameID;
	ODStorageUnitRef	suref;
	CAFrameRef			frameRef = 0;;
	
	THROW_IF_NULL (frame);

	if (frame->IsRoot(ev))
		return (kRootFrameRef);
	
	if (document)
	{
	
		rootPartSU = ((CADocument*)document)->GetRootPartSU();
		THROW_IF_NULL (rootPartSU);
		
		FocusDocForFrameRef (document);
	
		frameID = frame->GetID(ev);
		rootPartSU->GetStrongStorageUnitRef (ev, frameID, suref);
		ODBlockMove ((ODPtr) suref, (ODPtr) &frameRef, 4);
	}
	
	return (frameRef);
	
}

//-------------------------------------------------------------------------

CAFrameRef	GetRootFrameRef(	CADocumentRef	document)
{
	return (kRootFrameRef);
}



//-------------------------------------------------------------------------

ODFacet* FacetFromFrameRef (CADocumentRef document, CAFrameRef frame)
{
	Environment*			ev			= gCASession->GetEV();
	ODFrameFacetIterator*	iter;
	ODFacet*				targetFacet;

	TempODFrame tempFrame = AcquireFrameFromFrameRef (document, frame);
	THROW_IF_NULL (tempFrame);
	
	iter = tempFrame->CreateFacetIterator (ev);
	targetFacet = iter->First(ev);
		
	ODDeleteObject (iter);
		
	return (targetFacet);

}

//-------------------------------------------------------------------------

ODFrame*	GetRootFrame( CADocumentRef document )
{
	Environment*		ev			= gCASession->GetEV();
	ODFrame*			rootFrame	= kODNULL;

	if (((CADocument*)document)->GetDocumentType() == kCAProxyDocument)
	{
		if (((CADocument*)document)->GetWindow())
		{
			rootFrame = ((CADocument*)document)->GetWindow()->GetRootFrame(ev);
		}
	}
	
	return rootFrame;
}


Boolean		FrameRefValid		(CADocumentRef document, CAFrameRef frameRef)
{
	
	Environment*		ev			= gCASession->GetEV();
	Boolean				result		= false;
	ODStorageUnit*		rootPartSU;
	ODDraft*			theDraft;
	ODStorageUnitRef	suref;
	ODID				frameID;

	if (frameRef == kRootFrameRef) return true;
	
	TRY
	
		rootPartSU = ((CADocument*)document)->GetRootPartSU();
		THROW_IF_NULL (rootPartSU);
	
		theDraft = rootPartSU->GetDraft (ev);
		
		// Make sure the su is focused on something
		FocusDocForFrameRef (document);
		ODBlockMove ((ODPtr) &frameRef, suref, 4);
	
		frameID = rootPartSU->GetIDFromStorageUnitRef (ev, suref);
		TempODFrame tempFrame = theDraft->AcquireFrame (ev, frameID);
		
		if (tempFrame) result = true;
		
	CATCH_ALL
	ENDTRY
	
	return (result);

}


static void FocusDocForFrameRef (CADocumentRef document)
{

	Environment*			ev = gCASession->GetEV();
	ODStorageUnit*			rootSU;
	
	rootSU = ((CADocument*)document)->GetRootPartSU();
	THROW_IF_NULL (rootSU);
	
	if (((CADocument*)document)->GetDocumentType() == kCAProxyDocument)
		rootSU->Focus(ev, kCAPropEmbeddedFrames, kODPosUndefined, 
							kODNULL, 1, kODPosUndefined);
	else
	{
		ODSUForceFocus (ev, rootSU, kCAPropEmbeddedFrames, kODStrongStorageUnitRefs);
	}
	
}

