/*
	File:		CAInit.cpp

	Contains:	Container Application Library source

	Written by:	Tantek elik, Greg Ames, Rick Badertscher

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<12>	  8/2/95	RB		Renamed CASessn.cpp -> CAInit.cpp
		<11>	  5/6/95	SJF		WWDC bug fix session with Rick
		 <8>	  4/18/95	SJF		Cleanup...
		 <4+>	 2/27/95	SJF		Move CALibCFMInit here
		 <4>	 2/13/95	SJF		Interim checkin to update project database
		 <3>	  2/?/95	SJF		change CACreateDocument to use ProxyShell support
		 <2)	12/15/94	SJF		change somGetGlobalEnvironment to _gpProxyShell->GetGlobalEnvironment
		 <1>	10/30/94	GCA,DHN	All mods to make project compile with no
		 							errors under OpenDoc b1 (with SOM).
		 <0>	  8/9/94	T		first written
		 
	To Do:
		implement
*/

#ifndef _CASESSN_
#include "CASessn.h"
#endif

#ifndef _CAERROR_
#include "CAError.h"
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef __GESTALTEQU__
#include <GestaltEqu.h>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef _ODMEMORY_
#include <ODMemory.h>
#endif

#ifndef _USERSRCM_
#include <UseRsrcM.h>
#endif

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif


#ifndef _CADEBUG_
#include "CADebug.h"
#endif


#pragma segment CALib

extern DragSendDataUPP		gDragSendProc;

extern pascal OSErr	CALibSendDataProc(	FlavorType theType,
										void* refCon,
										ItemReference theItem,
										DragReference theDrag);

//-------------------------------------------------------------------------

pascal	void	CAInit( void )
{

	Environment*			ev		= somGetGlobalEnvironment();
	CASession*				session = kODNULL;	ODVolatile (session);

	WASSERT (!gCASession); // Make sure Init is only called once...

	ODInitExceptions();		// Sets up SOMError and SOMPrintf
	InitODMemory();
	
#if TRACING
	gCATracer = new TCATracer (kODNULL, true);
#endif

	gDragSendProc = NewDragSendDataProc(CALibSendDataProc);

	CA_TRY
	
		session = new CASession;
		THROW_IF_NULL(session);
		session->Initialize();
		
	CA_CATCH_ALL
	
		ODDeleteObject (session);
		session = kODNULL;
		
	CA_ENDTRY
	
	gCASession = session;
	
	
}

//-------------------------------------------------------------------------

pascal	void	CAShutdown( void )
{

#if TRACING
	delete gCATracer;
#endif

	if (gCASession)
		delete gCASession;

}


//-------------------------------------------------------------------------
//	CFM initialization for CALib 
//		Provides CALib with access to its resource fork when the library
//		gets opened.
//-------------------------------------------------------------------------

#define kCALibGestaltSelectorRsrcType	'GEST'
#define kCALibGestaltSelectorRsrcID		6969

extern "C" pascal OSErr CALibCFMInit( CFragInitBlockPtr );


pascal OSErr CALibCFMInit( CFragInitBlockPtr initBlkPtr )
{
	OSErr err;

	err = InitLibraryResources( initBlkPtr );
	if (err != noErr)
		DEBUGSTR_ASSERT( "\pCALibCFMInit: InitLibraryResources failed" );

	return err;
}
