/*
	File:		CASessn.cpp

	Contains:	CASession class Implementation

	Written by:	Rick Badertscher

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		<13>	  11/10/95	RB		Fixed up CASession::InstallMenuBar()
		<12>	  8/2/95	RB		Moved CALib public init functions to CAInit.cpp.
									CASessn is now a CALib private implementation file.
		<11>	  5/6/95	SJF		WWDC bug fix session with Rick
		 <8>	  4/18/95	SJF		Cleanup...
		 <4+>	 2/27/95	SJF		Move CALibCFMInit here
		 <4>	 2/13/95	SJF		Interim checkin to update project database
		 <3>	  2/?/95	SJF		change CACreateDocument to use ProxyShell support
		 <2)	12/15/94	SJF		change somGetGlobalEnvironment to _gpProxyShell->GetGlobalEnvironment
		 <1>	10/30/94	GCA,DHN	All mods to make project compile with no
		 							errors under OpenDoc b1 (with SOM).
		 <0>	  8/9/94	T		first written
		 
	To Do:
		implement
*/

#ifndef _CASESSN_
#include "CASessn.h"
#endif

#ifndef _CAUTIL_
#include "CAUtil.h"
#endif

#ifndef _CADISPTCH_
#include "CADisptch.h"
#endif

#ifndef _CADOCPRIV_
#include "CADocPriv.h"
#endif

#ifndef SOM_CAWindowState_xh
#include "CAWinStat.xh"
#endif

#ifndef _SIHELPER_
#include <SIHelper.h>
#endif

#ifndef SOM_ShellSI_xh
#include "ShellSI.xh"
#endif

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdDefs_defined
#include <StdDefs.xh>
#endif

#ifndef SOM_ODArbitrator_xh
#include <Arbitrat.xh>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef SOM_ODStorageUnit_xh
#include <StorageU.xh>
#endif

#ifndef SOM_ODDocument_xh
#include <Document.xh>
#endif

#ifndef SOM_ODContainer_xh
#include <ODCtr.xh>
#endif

#ifndef SOM_ODSession_xh
#include <ODSessn.xh>
#endif

#ifndef _ITEXT_
#include <IText.h>
#endif

#ifndef _USERSRCM_
#include <UseRsrcM.h>
#endif

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif

#ifndef _ORDCOLL_
#include <OrdColl.h>
#endif

#ifndef SOM_ODClipboard_xh
#include <Clipbd.xh>
#endif

#ifndef SOM_ODDragAndDrop_xh
#include "DragDrp.xh"
#endif

#ifndef SOM_ODDragItemIterator_xh
#include "DgItmIt.xh"
#endif

#ifndef SOM_ODMenuBar_xh
#include <MenuBar.xh>
#endif

#ifndef _STORUTIL_
#include <StorUtil.h>
#endif

#ifndef SOM_ODUndo_xh
#include <Undo.xh>
#endif

#ifndef SOM_Module_OpenDoc_Commands_defined
#include <CmdDefs.xh>
#endif

#ifndef _BARRAY_
#include <BArray.h>
#endif

#ifndef SOM_ODDispatcher_xh
#include <Disptch.xh>
#endif

#ifndef SOM_ODFoci_xh
#include <Foci.xh>
#endif

#ifndef _StdTypIO_
#include <StdTypIO.h>
#endif

#ifndef SOM_Module_OpenDoc_StdTypes_defined
#include <StdTypes.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdDefs_defined
#include <StdDefs.xh>
#endif

#ifndef _TEMPOBJ_
#include <TempObj.h>
#endif


#pragma segment CALib

//=========================================================================
// Instance variables - need to be hidden from CA Implementors
//=========================================================================

Boolean			_gCAInitted;
CASession*		gCASession;


//-------------------------------------------------------------------------------------
// Menus
//-------------------------------------------------------------------------------------

#define	kSHLMenuAbout	1

enum {
	kSHLMenuNew			=	1
	,kSHLMenuOpen
	,kSHLMenuOpenDocument
	,kSHLMenuInsert
	,kSHLMenuClose
	,kSHLMenuDeleteDocument

	// menu separator

	,kSHLMenuSave		=	8
	,kSHLMenuSaveACopy
	,kSHLMenuRevert
	,kSHLMenuDraft
	,kSHLMenuDocumentInfo
	
	// menu separator

	,kSHLMenuPageSetup	=	14
	,kSHLMenuPrint
};


enum {
	kSHLMenuUndo		=	1
	,kSHLMenuRedo
	
	// menu separator

	,kSHLMenuCut		=	4
	,kSHLMenuCopy
	,kSHLMenuPaste
	,kSHLMenuPasteAs
	,kSHLMenuClear
	,kSHLMenuSelectAll
	
	// menu separator

	,kSHLMenuGetPartInfo=	11
	,kSHLMenuPreferences

	// menu separator

	,kSHLMenuViewAsWin	=	14
};


#define 	kODAppleMenuID		128
#define 	kODDocumentMenuID	129
#define 	kODEditMenuID		130

extern DragSendDataUPP		gDragSendProc;

extern pascal OSErr	MySendDataProc(
					FlavorType theType,
					void* refCon,
					ItemReference theItem,
					DragReference theDrag);

//==============================================================================

CASession::CASession()
{
	fEV 						= kODNULL;	// Every document in this CA session shares fEV...
	fSession 					= kODNULL;
	fWindowState 				= kODNULL;
	fArbitrator 				= kODNULL;
	fFloatingWindowPart			= kODNULL;
	fFloatingWindowPartDraft	= kODNULL;
	fFloatingWindowCtrHandle	= kODNULL;
	fOpenCADocuments			= kODNULL;
	fCADragDropDocument			= kODNULL;
	fCAClipboardDocument		= kODNULL;
	fShellSemtInterface			= kODNULL;
}


CASession::~CASession()
{

	if (fFloatingWindowPart)
	{
		ODContainer*	tmpContainer;
		ODDocument*		tmpDocument;
				
		tmpDocument = fFloatingWindowPartDraft->GetDocument (fEV);
		tmpContainer = tmpDocument->GetContainer (fEV);
		
		ODReleaseObject( fEV, fFloatingWindowPart );
		ODReleaseObject( fEV, fFloatingWindowPartDraft );
		
		ODReleaseObject( fEV, tmpDocument );
		ODReleaseObject( fEV, tmpContainer );
		
		ODDisposeHandle(fFloatingWindowCtrHandle);
		fFloatingWindowCtrHandle = (ODHandle) kODNULL;

	}

#ifdef PATCHED_DRAGDROP
	ODDeleteObject (fCADragDropDocument);
#endif

	ODDeleteObject (fOpenCADocuments);
	ODDeleteObject (fCAClipboardDocument);
	ODDeleteObject (fCADispatcher);
	ODDeleteObject (fSession);
	

}


//-------------------------------------------------------------------------------------
// Initialization
//-------------------------------------------------------------------------------------

void		CASession::Initialize()
{

	CAWindowState*	windowState;
	
	fEV = somGetGlobalEnvironment();
		
	fSession = new ODSession;
	fSession->InitSession(fEV);
	
	// Patch the ODWindowState object
	windowState = new CAWindowState;
	windowState->InitCAWindowState (fEV, fSession);
	
	// Patch the ODDragAndDrop object
	
#ifdef PATCHED_DRAGDROP
	CADragAndDrop* dragDrop = new CADragAndDrop;
	dragDrop->InitDragAndDrop (fEV, fSession);
#endif

	fWindowState = fSession->GetWindowState( fEV );
	fArbitrator = fSession->GetArbitrator( fEV );
	
	// Create the CADispatcher object
	fCADispatcher = new CADispatcher;
	fCADispatcher->Initialize(fSession);

	fOpenCADocuments = new OrderedCollection;
	
	// Attach a dummy semantic interface extension
	fShellSemtInterface = new ShellSI;
	fSession->SetShellSemtInterface(fEV, fShellSemtInterface);
	SIHelper* semItfcHelper = new SIHelper;
	semItfcHelper->InitSIHelper(fSession, fShellSemtInterface);
	fShellSemtInterface->InitCPlusSemanticInterface(fEV, kODAppShell, semItfcHelper,
													fSession);


}

//-------------------------------------------------------------------------------------

void 		CASession::InstallMenuBar(ODPlatformMenuBar appMenuBar)
{
	ODMenuBar*			theCAMenuBar	= kODNULL;
	
	// appMenuBar contains the desired base menu bar (Apple, File, Edit)
	
	SetMenuBar (appMenuBar);

	TempODMenuBar baseMenuBar = fWindowState->CreateMenuBar (fEV, GetMenuBar());

	MenuHandle appleMenu = GetMenuHandle(kODAppleMenuID);
	DetachResource ((Handle)appleMenu);
	
	if (appleMenu)
	{
		GetMenuItemText(	appleMenu, kSHLMenuAbout, fDefaultAboutMenuItem.text);
		GetItemCmd(			appleMenu, kSHLMenuAbout, &fDefaultAboutMenuItem.cmdChar);
		GetItemIcon(		appleMenu, kSHLMenuAbout, &fDefaultAboutMenuItem.iconID);
		GetItemMark(		appleMenu, kSHLMenuAbout, &fDefaultAboutMenuItem.markChar);
		GetItemStyle(		appleMenu, kSHLMenuAbout, &fDefaultAboutMenuItem.textStyle);
	}
	
	MenuHandle documentMenu = GetMenu(kODDocumentMenuID);
	DetachResource((Handle)documentMenu);
	baseMenuBar->AddMenuLast(fEV, kODDocumentMenuID, documentMenu, kODNULL);

	MenuHandle editMenu = GetMenu(kODEditMenuID);
	DetachResource((Handle)editMenu);
	baseMenuBar->AddMenuLast(fEV, kODEditMenuID, editMenu, kODNULL);

	if (editMenu)
	{
		// Save the default undo stuff
		
		GetMenuItemText(	editMenu, kSHLMenuUndo, fDefaultUndoMenuItem.text);
		GetItemCmd(			editMenu, kSHLMenuUndo, &fDefaultUndoMenuItem.cmdChar);
		GetItemIcon(		editMenu, kSHLMenuUndo, &fDefaultUndoMenuItem.iconID);
		GetItemMark(		editMenu, kSHLMenuUndo, &fDefaultUndoMenuItem.markChar);
		GetItemStyle(		editMenu, kSHLMenuUndo, &fDefaultUndoMenuItem.textStyle);
	
		GetMenuItemText(	editMenu, kSHLMenuRedo, fDefaultRedoMenuItem.text);
		GetItemCmd(			editMenu, kSHLMenuRedo, &fDefaultRedoMenuItem.cmdChar);
		GetItemIcon(		editMenu, kSHLMenuRedo, &fDefaultRedoMenuItem.iconID);
		GetItemMark(		editMenu, kSHLMenuRedo, &fDefaultRedoMenuItem.markChar);
		GetItemStyle(		editMenu, kSHLMenuRedo, &fDefaultRedoMenuItem.textStyle);
	}

	fWindowState->SetBaseMenuBar( fEV, baseMenuBar);

	baseMenuBar->Display(fEV);

	
}


//-------------------------------------------------------------------------------------

ODPart*			CASession::GetFloatingWindowHandlerPart (void)
{
	if (!fFloatingWindowPart)
	{
	
		// Construct an instance of NoPart as the fFloatingWindowPart

		ODContainer*	floatingWindowContainer;
		ODDocument*		floatingWindowDocument;
	
		fFloatingWindowCtrHandle = ODNewHandle(0);

		floatingWindowContainer = CreateMemoryContainer(fEV, fSession, fFloatingWindowCtrHandle, kODBentoMemoryContainer);
		floatingWindowDocument = floatingWindowContainer->AcquireDocument(fEV, kODDefaultDocument);
		fFloatingWindowPartDraft = floatingWindowDocument->AcquireBaseDraft( fEV, kODDPExclusiveWrite );
		fFloatingWindowPart = fFloatingWindowPartDraft->CreatePart( fEV, kODBlackBoxHandlerOfLastResort, kODNULL);
		
		ODStorageUnit* su = fFloatingWindowPartDraft->AcquireDraftProperties( fEV );
		ODSetStrongSURefProp(fEV, su, kODPropRootPartSU, kODStrongStorageUnitRef,
						fFloatingWindowPart->GetStorageUnit( fEV )->GetID( fEV ));
		ODReleaseObject (fEV, su);
		
	}
	
	return (fFloatingWindowPart);
}

//-------------------------------------------------------------------------------------

void CASession::AddCADocumentRef (CADocument*	docRef)
{
	
	fOpenCADocuments->AddLast (docRef);
}

//-------------------------------------------------------------------------------------

void CASession::RemoveCADocumentRef (CADocument*	docRef)
{
	fOpenCADocuments->Remove (docRef);
}


//-------------------------------------------------------------------------------------

Boolean CASession::HaveCADocumentRef(CADocumentRef document)
{

	return (fOpenCADocuments->Contains ((CADocument*) document));
	
}


//-------------------------------------------------------------------------------------

CADocument* CASession::FindCADocumentRefForDraft(ODDraft* draft)
{
	OrderedCollectionIterator*	iter;
 	CADocument*					aDoc;
 	Boolean						matched = kODFalse;
 	
  	iter = fOpenCADocuments->CreateIterator();

	for (aDoc = (CADocument*)iter->First();
				iter->IsNotComplete(); 
				aDoc = (CADocument*)iter->Next())
	{
		if (aDoc->GetDraft() == draft)
		{
			matched = kODTrue;
			break;
		}
		
	}
	
	ODDeleteObject (iter);
	
	if (!matched)
	{
		aDoc = kODNULL;
	}
	
	return (aDoc);

}

//-------------------------------------------------------------------------------------

CADocument* CASession::FindCADocumentRefForWindow (ODWindow* window)
{
	OrderedCollectionIterator*	iter;
 	CADocument*				aDoc;
 	Boolean						matched = kODFalse;
 	
  	iter = fOpenCADocuments->CreateIterator();

	for (aDoc = (CADocument*)iter->First();
				iter->IsNotComplete(); 
				aDoc = (CADocument*)iter->Next())
	{
		if (aDoc->GetWindow() == window)
		{
			matched = kODTrue;
			break;
		}
		
	}
	
	ODDeleteObject (iter);
	
	if (!matched)
	{
		aDoc = kODNULL;
	}
	
	return (aDoc);
	
}

//-------------------------------------------------------------------------------------

CAClipboardDocument*	CASession::GetCAClipboardDocument (	void)
{
	Environment*		ev = gCASession->GetEV();
	ODClipboard*		odClipboard;
	
	if (fCAClipboardDocument == NULL)
	{
		odClipboard = this->GetODSession()->GetClipboard(ev);
		fCAClipboardDocument = new CAClipboardDocument (odClipboard);
	}
	
	return (fCAClipboardDocument);
			
}

//-------------------------------------------------------------------------------------
CADragDropDocument*	CASession::GetCADragDropDocument (	void)
{
	Environment*		ev = gCASession->GetEV();
	ODDragAndDrop*		odDragDrop;
	
	if (fCADragDropDocument == NULL)
	{
		odDragDrop = this->GetODSession()->GetDragAndDrop(ev);
		fCADragDropDocument = new CADragDropDocument (odDragDrop);
	}
	
	return (fCADragDropDocument);
			
}

//-------------------------------------------------------------------------------------
// CASession::UpdateMenus
//-------------------------------------------------------------------------------------

void CASession::UpdateMenus()
{

	MenuHandle		appleMenu;
	Boolean			isCAProxyPart;
	
	// Determine if the CA has the menu focus
	TempODFrame tempFrame = fArbitrator->AcquireFocusOwner (fEV,
		gCASession->GetODSession()->Tokenize(fEV, kODMenuFocus));
		
	if (!tempFrame) return;
	
	TempODPart tempPart = tempFrame->AcquirePart(fEV);
	
	isCAProxyPart = IsCAProxyPart (tempPart);
	
	if (isCAProxyPart)
	{
	
		TempODMenuBar tempMenuBar = fWindowState->AcquireBaseMenuBar(fEV);
	
		appleMenu = tempMenuBar->GetMenu (fEV, kODAppleMenuID);
	
		SetMenuItemText(	appleMenu, kSHLMenuAbout, fDefaultAboutMenuItem.text);
		SetItemCmd(			appleMenu, kSHLMenuAbout, fDefaultAboutMenuItem.cmdChar);
		SetItemIcon(		appleMenu, kSHLMenuAbout, fDefaultAboutMenuItem.iconID);
		SetItemMark(		appleMenu, kSHLMenuAbout, fDefaultAboutMenuItem.markChar);
		SetItemStyle(		appleMenu, kSHLMenuAbout, fDefaultAboutMenuItem.textStyle);
	
		this->UpdateUndoMenus();
		
	}
	else
	{
		this->UpdateUndoMenus();
		fWindowState->AdjustPartMenus (fEV);
	}
	
}

//-------------------------------------------------------------------------------------
// CASession::UpdateUndoMenus
//-------------------------------------------------------------------------------------

void CASession::UpdateUndoMenus()
{
	ODUndo*			undo = fSession->GetUndo(fEV);
	ODPart*			part;
	ODActionData	actionData;
	ODActionType	actionType;
	ODName			actionLabel;
	MenuHandle		editMenu;
	
	TempODMenuBar tempMenuBar = fWindowState->AcquireBaseMenuBar(fEV);

	editMenu = tempMenuBar->GetMenu(fEV, kODEditMenuID);

	ODVolatile(actionLabel);
	ODVolatile(actionData);

	actionLabel.text._buffer = kODNULL;
	actionData._buffer = kODNULL;

	
	TRY
		if (undo->PeekUndoHistory(fEV, &part, &actionData, &actionType,
									&actionLabel))
		{
			tempMenuBar->EnableCommand(fEV, kODCommandUndo, kODTrue);
			tempMenuBar->SetItemString(fEV, kODCommandUndo, &actionLabel);
		}
		else
		{
			tempMenuBar->EnableCommand(fEV, kODCommandUndo, kODFalse);
			SetMenuItemText(editMenu, kSHLMenuUndo, fDefaultUndoMenuItem.text);
			SetItemCmd(editMenu, kSHLMenuUndo, fDefaultUndoMenuItem.cmdChar);
			SetItemIcon(editMenu, kSHLMenuUndo, fDefaultUndoMenuItem.iconID);
			SetItemMark(editMenu, kSHLMenuUndo, fDefaultUndoMenuItem.markChar);
			SetItemStyle(editMenu, kSHLMenuUndo, fDefaultUndoMenuItem.textStyle);

			//this->ResetUndoText(editMenu);
		}
	
		DisposeITextStruct(actionLabel);
		DisposeByteArrayStruct(actionData);
		actionLabel.text._buffer = kODNULL;
		actionData._buffer = kODNULL;

		if (undo->PeekRedoHistory(fEV, &part, &actionData, &actionType,
									&actionLabel))
		{
			tempMenuBar->EnableCommand(fEV, kODCommandRedo, kODTrue);
			tempMenuBar->SetItemString(fEV, kODCommandRedo, &actionLabel);
		}
		else
		{
			tempMenuBar->EnableCommand(fEV, kODCommandRedo, kODFalse);
			SetMenuItemText(editMenu, kSHLMenuRedo, fDefaultRedoMenuItem.text);
			SetItemCmd(editMenu, kSHLMenuRedo, fDefaultRedoMenuItem.cmdChar);
			SetItemIcon(editMenu, kSHLMenuRedo, fDefaultRedoMenuItem.iconID);
			SetItemMark(editMenu, kSHLMenuRedo, fDefaultRedoMenuItem.markChar);
			SetItemStyle(editMenu, kSHLMenuRedo, fDefaultRedoMenuItem.textStyle);

			//this->ResetRedoText(editMenu);
		}

	CATCH_ALL
		if (actionLabel.text._buffer)
			DisposeITextStruct(actionLabel);
		if (actionData._buffer)
			DisposeByteArrayStruct(actionData);
		RERAISE;
	ENDTRY

	DisposeITextStruct(actionLabel);
	DisposeByteArrayStruct(actionData);
}

//-------------------------------------------------------------------------------------
// CASession::RegisterCommand
//-------------------------------------------------------------------------------------

void CASession::RegisterCommand(	CACommandID		command,
									CAMenuID		menu,
									CAMenuItemID	menuItem)
{
	TempODMenuBar tempMenuBar = fWindowState->AcquireBaseMenuBar(fEV);
	tempMenuBar->RegisterCommand (fEV, command, menu, menuItem);	
}


//-------------------------------------------------------------------------------------
// CASession::HandleMenuCommand
//-------------------------------------------------------------------------------------

Boolean CASession::HandleMenuCommand(ODSLong menuResult, ODEventData* event)
{
	ODUnused(event);

	ODSShort		menu		= HiWord(menuResult);
	ODSShort 		item		= LoWord(menuResult);
	Boolean			result		= false;
	ODBoolean		isCAWindow	= true;
	ODCommandID 	command;

	TempODMenuBar tempMenuBar = fWindowState->AcquireBaseMenuBar(fEV);
	command = tempMenuBar->GetCommand(fEV, menu, item);

	if (tempMenuBar->IsCommandSynthetic(fEV,command))
	{
		switch (menu)
		{
		
			case kODAppleMenuID: // Apple
			{
				Str255		daName;
				ODSShort	daRefNum;
			
				GetMenuItemText(GetMenuHandle(kODAppleMenuID), item, daName);
				daRefNum = OpenDeskAcc(daName);
				HiliteMenu(0);
				break;
			}
		
		
			default:	 break;
		}
	}
	else 
	{
		ODWindow* activeWindow = fWindowState->AcquireActiveWindow(fEV);
		
		if (!gCASession->FindCADocumentRefForWindow (activeWindow))
			isCAWindow = false;

		switch(command)
		{
			case kODCommandUndo:	fSession->GetUndo(fEV)->Undo(fEV); result = true;	break;
			case kODCommandRedo:	fSession->GetUndo(fEV)->Redo(fEV); result = true;	break;
			
			case kODCommandClose:
			
				if (!isCAWindow)
				{
					activeWindow->CloseAndRemove(fEV);
					activeWindow = kODNULL;	
					result = true;		
				}

				break;
				
			default:
				break;
		}
		
		ODRelease (fEV, activeWindow);
		HiliteMenu(0);
	} 
		
	return (result);
	
}

