/*
	File:		CAUtil.cpp

	Contains:	CALib Utility routine implementations

	Written by:	Rick Badertscher

	Copyright:	 1994 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

		 <0>	8/17/95	RB		SOMverted
	To Do:
*/

#ifndef _CAUTIL_
#include "CAUtil.h"
#endif

#ifndef _CADOCPRIV_
#include "CADocPriv.h"
#endif

#ifndef _CAFRMUTL_
#include "CAFrmUtl.h"
#endif

#ifndef SOM_ODShape_xh
#include <Shape.xh>
#endif

#ifndef SOM_ODDraft_xh
#include <Draft.xh>
#endif

#ifndef _ODUTILS_
#include <ODUtils.h>
#endif

#ifndef CAProxyExtension_API
#include "CAProxyExtension.xh"
#endif

#ifndef _CAPROXYPARTDEF_
#include "CAProxyPartDef.h"
#endif

#ifndef SOM_ODPart_xh
#include <Part.xh>
#endif

#ifndef _StdTypIO_
#include <StdTypIO.h>
#endif

#ifndef _INFOUTIL_
#include <InfoUtil.h>
#endif

#ifndef _ISOSTR_
#include <ISOStr.h>
#endif

#ifndef _TEMPOBJ_
#include <TempObj.h>
#endif

#ifndef SOM_Module_OpenDoc_StdTypes_defined
#include <StdTypes.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdProps_defined
#include <StdProps.xh>
#endif

#ifndef SOM_Module_OpenDoc_StdDefs_defined
#include <StdDefs.xh>
#endif


#pragma segment CALib



//-------------------------------------------------------------------------

void CallProxyAddFrame (CADocumentRef document, ODFrame* frame)
{
	Environment*		ev			= gCASession->GetEV();
	CAProxyExtension*	proxyExt	= kODNULL;
	CADocument*			docRef		= kODNULL;
	
	docRef = (CADocument*) document;
	
	TempODPart tempPart = ((CADocument*)document)->AcquireRootPart();
	THROW_IF_NULL (tempPart);
	
	proxyExt = (CAProxyExtension*)tempPart->AcquireExtension( ev, kCAProxyPartExtension );
	THROW_IF_NULL (proxyExt);
	
	proxyExt->AddFrame( ev, frame);
	
	ODReleaseObject( ev, proxyExt );


}

//-------------------------------------------------------------------------
void		GetEventInfo(CADocumentRef document, ODEventInfo* eventInfo )
{
	Environment*		ev			= gCASession->GetEV();
	CAProxyExtension*	proxyExt	= kODNULL;
		
	TempODPart tempPart = ((CADocument*)document)->AcquireRootPart();
	THROW_IF_NULL (tempPart);
	
	proxyExt = (CAProxyExtension*)tempPart->AcquireExtension( ev, kCAProxyPartExtension );
	THROW_IF_NULL (proxyExt);
	
	proxyExt->GetLastEventInfo( ev, eventInfo);
	proxyExt->Release(ev);
		
}


//-------------------------------------------------------------------------

Boolean IsCAProxyPartDraft (ODDraft* draft)
{
	Environment*		ev					= gCASession->GetEV();
	Boolean				retVal				= false;
	ODID				suID;
	ODType				kind;
	
	TempODStorageUnit tempPropsSU = draft->AcquireDraftProperties(ev);
	suID = ODGetStrongSURefProp(ev, tempPropsSU, kODPropRootPartSU, kODStrongStorageUnitRef);
		
	TempODStorageUnit tempRootSU = draft->AcquireStorageUnit(ev, suID);
	kind = ODGetKindFromPartSU(ev, tempRootSU);
	
	if (!ODISOStrCompare (kind, kCAProxyPartKind))
	{
		retVal = true;
	}
	
	return (retVal);

}


//-------------------------------------------------------------------------

pascal void ProxyHook_FrameShapeRequest (	CAFrameShapeRequestHandler handler,
											ODDraft*			draft,
											ODFrame*			frame,
											ODShape*			shape)
{
	Environment*	ev = gCASession->GetEV();
	ODRgnHandle		rgn = kODNULL;
	CAFrameRef		frameRef;
	ODBoolean		adjusted = kODFalse;
	CADocument*		docRef;

	docRef = gCASession->FindCADocumentRefForDraft (draft);
	
	CA_TRY
	
		CAFrameShapeRequestHandlerUPP theHandler;
		theHandler = (CAFrameShapeRequestHandlerUPP) NewRoutineDescriptor((ProcPtr)handler, 
												uppFrameShapeRequestHandlerInfo, 
												GetCurrentISA());


		frameRef = FrameRefFromFrame ((CADocumentRef)docRef, frame);
		rgn = shape->CopyQDRegion(ev);
		
		if (rgn)
		{
			adjusted = (Boolean) CallUniversalProc(theHandler, uppFrameShapeRequestHandlerInfo,
								(CADocumentRef)docRef, frameRef, rgn);
		}
	

		DisposeRoutineDescriptor (theHandler);
		
	CA_CATCH_ALL
	CA_ENDTRY
	
	if (adjusted)
		shape->SetQDRegion (ev, rgn);
	else
		DisposeRgn (rgn);

	return;

}


//-------------------------------------------------------------------------

pascal void ProxyHook_AdjustBorder (	CAAdjustBorderProc adjustBorderProc,
										ODDraft* draft,
										ODFrame* frame,
										ODShape* shape)
{
	Environment*	ev = gCASession->GetEV();
	RgnHandle		rgn = NULL;
	CAFrameRef		frameRef;
	Boolean			adjusted = kODFalse;
	CADocumentRef	docRef;


	CA_TRY
	
		CAAdjustBorderProcUPP theHandler;
		theHandler = (CAAdjustBorderProcUPP) NewRoutineDescriptor((ProcPtr)adjustBorderProc, 
												uppCAAdjustBorderProcInfo, 
												GetCurrentISA());

		docRef = (CADocumentRef) gCASession->FindCADocumentRefForDraft (draft);

		frameRef = FrameRefFromFrame (docRef, frame);
		rgn = shape->CopyQDRegion(ev);
	
		adjusted = (Boolean) CallUniversalProc(theHandler, uppCAAdjustBorderProcInfo,
								docRef, frameRef, rgn);

		DisposeRoutineDescriptor (theHandler);

	CA_CATCH_ALL
	CA_ENDTRY
	
	
	if (adjusted)
		shape->SetQDRegion (ev, rgn);
	else
		DisposeRgn (rgn);
	
	return;

}

