/*
	File:		FacetLink.cpp

	Contains:	Linked list of facets, for use by parts

	Written by:	Jens Alfke

	Change History (most recent first):

		 <3+>	 2/13/95	SJF		Place code in CAProxyPart segment
		 <3>	 2/13/95	SJF		Interim checkin to update project database
		 <0>	11/16/94	SJF		first written
		 
	To Do:
		implement

*/

#define DEBUGSTR_API(A)		/* */
#define DEBUGSTR_ASSERT(A)	/* */

#ifndef _FACETLIST_
#include "FacetList.h"
#endif

#ifndef _ODTYPES_
#include <ODTypes.h>	// From CodeWarrior:Headers :System Extras Headers :OpenDoc:Interfaces:Handmade:
#endif

//#ifdef applec
#pragma segment CAProxyPart
//#endif


//==============================================================================
// FacetLink
//==============================================================================


FacetLink::FacetLink( )
{
	fFacet = kODNULL;
	fPrev = this;
	fNext = this;
}


FacetLink::FacetLink( ODFacet *facet, FacetLink *list )
{
	fFacet = facet;
	fPrev  = list;
	fNext  = list->fNext;
	list->fNext  = this;
	fNext->fPrev = this;
}


FacetLink::~FacetLink( )
{
	if( fPrev )
		fPrev->fNext = fNext;
	if( fNext )
		fNext->fPrev = fPrev;
}


//==============================================================================
// FacetLink
//==============================================================================

FacetList::FacetList()
{
	fFacet = kODNULL;
	fPrev = kODNULL;
	fNext = this;
}

FacetList::~FacetList( )
{
	// Delete all links:
	while( fNext != this )
		delete fNext;
}

void FacetList::Add( ODFacet *facet )
{
	new FacetLink(facet,this);
	// The new link has already hooked itself into my chain so I don't need to
	// remember it elsewhere.
}


void FacetList::Remove( ODFacet *facet )
{
	FacetLink *link;
	for ( link = this->First(); link->Facet(); link = link->Next() ) {
		if( link->Facet() == facet ) {
			delete link;
			return;
		}
	}

	DEBUGSTR_ASSERT("\pCouldn't find facet");
}


